/* sanebuttond -- SANE button monitoring daemon
   Using the SANE library.
   Copyright (C) 2004 Rene Rebe, rene@rocklinux.org
   Copyright (C) 2005 Gerhard Jger, gerhard@gjaeger.de
   Copyright (C) 2007 Jean-Christophe Cardot, jice@cardot.net

   Code based on Rene Rebes' scanbuttond.cc
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program;a if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

/* Compile with:
   gcc `sane-config --libs -cflags` sane-nb-buttons.c -o sane-nb-buttons */

#include <assert.h>
#include <getopt.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <sane/sane.h>

#define MAX_BUTTONS 20

static SANE_Handle device;

void
usage(void)
{
	puts("Usage: sane-nb-buttons [--debug] [--help] [sane device name]");
	puts("Options: --debug: debug output");
	puts("         --help:  this help page");
	exit(1);
}

int
main (int argc, char **argv)
{
	int i, idx, nbbuttons;
	int debug = 0;
	const SANE_Option_Descriptor *opt;
	const SANE_Device **device_list;
	SANE_Int val;
	SANE_Int num_dev_options = 0;
	SANE_Int button_list[MAX_BUTTONS];

	const char *devname = 0;
	char sane_opt_type[7];
	char *c;

	SANE_Status status;
	SANE_Int version_code;

	sane_init (&version_code, 0);

	/* simple argument parsing */
	for (argv++; *argv; argv++)
	{
		if (!strcmp(*argv, "--debug")) {
			debug = 1;
		}
		else if (!strcmp(*argv, "--help")) {
			usage();
		}
		else if (!devname) {
			devname = *argv;
		}
		else {
			fprintf(stderr, "Too many arguments: %s\n", *argv);
			usage();
		}
	}

	if (!devname) {
		/* If no device name was specified explicitly, we look at the
		 environment variable SANE_DEFAULT_DEVICE.  If this variable
		 is not set, we open the first device we find (if any): 
		*/
		devname = getenv ("SANE_DEFAULT_DEVICE");
		if (!devname) {
			printf( "Scanning for devices ...\n" );
			status = sane_get_devices (&device_list, SANE_FALSE);
			if (status != SANE_STATUS_GOOD) {
				fprintf( stderr, "sane_get_devices() failed: %s\n", 
				         sane_strstatus(status));
				return 1;
			}
			if (!device_list[0]) {
				fprintf( stderr, " no SANE devices found!\n" );
				return 1;
			}

			printf( "%s %s %s %s\n", device_list[0]->name, device_list[0]->vendor,
			        device_list[0]->model, device_list[0]->type );

			devname = device_list[0]->name;
		}
	}
  
	// now we should have a valid devname
	status = sane_open (devname, &device);
	if (status != SANE_STATUS_GOOD) {
		fprintf( stderr, "open of device %s failed: %s\n", 
		         devname, sane_strstatus(status));
		return 1;
	}
  
	if (!device) {
		fprintf( stderr, "No SANE device detected! %s\n", 
		         sane_strstatus (status));
		return 1;
	}
 
	/* We got a device, find out how many options it has: */
	status = sane_control_option (device, 0, SANE_ACTION_GET_VALUE,
				&num_dev_options, 0);
	if (status != SANE_STATUS_GOOD) {
		fprintf( stderr, "unable to determine option count\n" );
		return 1;
	}

	idx=0;
	memset( button_list, -1, sizeof(button_list));
	for ( i = 0; i < num_dev_options; ++i) {
		opt = sane_get_option_descriptor (device, i);
		if (opt->name ) {
			if( SANE_OPTION_IS_ACTIVE(opt->cap)) {
			    if ( !strncmp( opt->name, "button", 6)) {
					button_list[idx++] = i;
					if ( debug ) {
						switch ( opt->type ) {
							case SANE_TYPE_BOOL:
								strcpy(sane_opt_type, "BOOL"); break;
                					case SANE_TYPE_INT:
								strcpy(sane_opt_type, "INT"); break;
							case SANE_TYPE_FIXED:
								strcpy(sane_opt_type, "FIXED"); break;
							case SANE_TYPE_STRING:
								strcpy(sane_opt_type, "STRING"); break;
							case SANE_TYPE_BUTTON:
								strcpy(sane_opt_type, "BUTTON"); break;
							case SANE_TYPE_GROUP:
								strcpy(sane_opt_type, "GROUP"); break;
						}
						printf(" - OPTION[%u], NAME: %s (%s), DESC: %s, TYPE: %s, button %u\n", i, opt->name, 
						   SANE_OPTION_IS_ACTIVE(opt->cap)?"ACTIVE":"NOT ACTIVE",
						   opt->desc, sane_opt_type, idx - 1 );
					}
				}
			}
		}
	}
	if ( debug && (idx == 0) ) {
		fprintf( stderr, "Device has no buttons, bye...\n" );
		return 1;
	}
	else if ( debug ) {
		printf( "We have found %u buttons!\n", idx );
	} else {
		printf( "%u", idx );
	}

	return 0;
}
